@@ -19,9 +19,8 @@ Handles Jingle contents (XEP 0166)
 from typing import Any  # pylint: disable=unused-import
 from typing import Dict  # pylint: disable=unused-import
 
-import os
-
 import nbxmpp
+from nbxmpp.namespaces import Namespace
 
 from gajim.common import app
 from gajim.common import configpaths
@@ -88,7 +87,7 @@ class JingleContent:
             'session-terminate': [],
             'transport-info': [self.__on_transport_info],
             'transport-replace': [self.__on_transport_replace],
-            'transport-accept': [],
+            'transport-accept': [self.__on_transport_replace],
             'transport-reject': [],
             'iq-result': [],
             'iq-error': [],
@@ -185,7 +184,7 @@ class JingleContent:
 
     def _fill_content(self, content):
         description_node = nbxmpp.simplexml.Node(
-            tag=nbxmpp.NS_JINGLE_FILE_TRANSFER_5 + ' description')
+            tag=Namespace.JINGLE_FILE_TRANSFER_5 + ' description')
         file_tag = description_node.setTag('file')
         if self.file_props.name:
             node = nbxmpp.simplexml.Node(tag='name')
@@ -202,7 +201,7 @@ class JingleContent:
         if self.file_props.type_ == 'r':
             if self.file_props.hash_:
                 file_tag.addChild('hash', attrs={'algo': self.file_props.algo},
-                                  namespace=nbxmpp.NS_HASHES_2,
+                                  namespace=Namespace.HASHES_2,
                                   payload=self.file_props.hash_)
         else:
             # if the file is less than 10 mb, then it is small
@@ -219,23 +218,19 @@ class JingleContent:
                              'date' : self.file_props.date,
                              'peerjid' : pjid
                             }
-                self.session.connection.set_file_info(file_info)
+                self.session.connection.get_module('Jingle').set_file_info(file_info)
         desc = file_tag.setTag('desc')
         if self.file_props.desc:
             desc.setData(self.file_props.desc)
         if self.use_security:
             security = nbxmpp.simplexml.Node(
-                tag=nbxmpp.NS_JINGLE_XTLS + ' security')
-            certpath = os.path.join(
-                configpaths.get('MY_CERT'), SELF_SIGNED_CERTIFICATE) + '.cert'
+                tag=Namespace.JINGLE_XTLS + ' security')
+            certpath = configpaths.get('MY_CERT') / (SELF_SIGNED_CERTIFICATE
+                                                     + '.cert')
             cert = load_cert_file(certpath)
             if cert:
-                try:
-                    digest_algo = (cert.get_signature_algorithm()
-                                   .decode('utf-8').split('With')[0])
-                except AttributeError:
-                    # Old py-OpenSSL is missing get_signature_algorithm
-                    digest_algo = "sha256"
+                digest_algo = (cert.get_signature_algorithm()
+                               .decode('utf-8').split('With')[0])
                 security.addChild('fingerprint').addData(cert.digest(
                     digest_algo).decode('utf-8'))
                 for m in ('x509', ): # supported authentication methods
