@@ -40,6 +40,7 @@ MODULES = [
     'bookmarks',
     'caps',
     'carbons',
+    'chat_markers',
     'chatstates',
     'delimiter',
     'discovery',
@@ -70,6 +71,7 @@ MODULES = [
     'user_mood',
     'user_nickname',
     'user_tune',
+    'vcard4',
     'vcard_avatars',
     'vcard_temp',
     'announce',
