@@ -17,7 +17,7 @@ from collections import namedtuple
 from datetime import timedelta
 
 import nbxmpp
-from nbxmpp.util import is_error_result
+from nbxmpp.errors import StanzaError
 from nbxmpp.namespaces import Namespace
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -28,12 +28,11 @@ from gajim.common import ged
 from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 
-from gajim.gtk.util import ensure_not_destroyed
-from gajim.gtk.util import get_builder
-from gajim.gtk.util import EventHelper
-from gajim.gtk.util import open_window
+from .util import get_builder
+from .util import EventHelper
+from .util import open_window
 
-log = logging.getLogger('gajim.gtk.server_info')
+log = logging.getLogger('gajim.gui.server_info')
 
 
 class ServerInfo(Gtk.ApplicationWindow, EventHelper):
@@ -46,6 +45,7 @@ class ServerInfo(Gtk.ApplicationWindow, EventHelper):
         self.set_default_size(400, 600)
         self.set_show_menubar(False)
         self.set_title(_('Server Info'))
+        self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
 
         self.account = account
         self._destroyed = False
@@ -232,12 +232,14 @@ class ServerInfo(Gtk.ApplicationWindow, EventHelper):
                 'days': delta.days, 'hours': hours}
             self._ui.server_uptime.set_text(uptime)
 
-    @ensure_not_destroyed
-    def _software_version_received(self, result):
-        if is_error_result(result):
+    def _software_version_received(self, task):
+        try:
+            result = task.finish()
+        except StanzaError:
             self.version = _('Unknown')
         else:
             self.version = '%s %s' % (result.name, result.version)
+
         self._ui.server_software.set_text(self.version)
 
     @staticmethod
@@ -298,9 +300,13 @@ class ServerInfo(Gtk.ApplicationWindow, EventHelper):
                     con.get_module('HTTPUpload').available,
                     http_upload_info),
             Feature('XEP-0398: Avatar Conversion',
-                    con.avatar_conversion),
+                    con.get_module('VCardAvatars').avatar_conversion_available),
             Feature('XEP-0411: Bookmarks Conversion',
-                    con.get_module('Bookmarks').conversion)
+                    con.get_module('Bookmarks').conversion),
+            Feature('XEP-0402: Bookmarks Compat',
+                    con.get_module('Bookmarks').compat),
+            Feature('XEP-0402: Bookmarks Compat PEP',
+                    con.get_module('Bookmarks').compat_pep)
         ]
 
     def _on_clipboard_button_clicked(self, _widget):
