@@ -22,13 +22,12 @@ from gajim.common import app
 from gajim.common import helpers
 from gajim.common.helpers import is_affiliation_change_allowed
 from gajim.common.helpers import is_role_change_allowed
-from gajim.common.i18n import ngettext
 from gajim.common.i18n import _
 from gajim.common.const import URIType
 from gajim.common.const import URIAction
 
-from gajim.gtk.util import get_builder
-from gajim.gtk.const import ControlType
+from gajim.gui.util import get_builder
+from gajim.gui.const import ControlType
 
 
 def build_resources_submenu(contacts, account, action, room_jid=None,
@@ -146,7 +145,7 @@ show_bookmarked=False, force_resource=False):
     item = Gtk.SeparatorMenuItem.new() # separator
     invite_to_submenu.append(item)
     for (room_jid, account) in rooms2:
-        menuitem = Gtk.MenuItem.new_with_label(room_jid.getNode())
+        menuitem = Gtk.MenuItem.new_with_label(room_jid.localpart)
         if len(contact_list) > 1: # several resources
             menuitem.set_submenu(build_resources_submenu(
                 contact_list, account, roster.on_invite_to_room, str(room_jid),
@@ -291,7 +290,7 @@ control=None, gc_contact=None, is_anonymous=True):
         item.hide()
 
     # Zeroconf Account
-    if app.config.get_per('accounts', account, 'is_zeroconf'):
+    if app.settings.get_account_setting(account, 'is_zeroconf'):
         for item in (send_single_message_menuitem,
         invite_menuitem, block_menuitem, unblock_menuitem, ignore_menuitem,
         unignore_menuitem, subscription_menuitem,
@@ -512,21 +511,21 @@ def get_transport_menu(contact, account):
     return menu
 
 
-def get_singlechat_menu(control_id, account, jid):
+def get_singlechat_menu(control_id, account, jid, type_):
     singlechat_menu = [
         (_('Send File'), [
             ('win.send-file-httpupload-', _('Upload File…')),
             ('win.send-file-jingle-', _('Send File Directly…')),
             ]),
+        ('win.send-marker-', _('Send Read Markers')),
         (_('Send Chatstate'), ['chatstate']),
         ('win.invite-contacts-', _('Invite Contacts…')),
         ('win.add-to-roster-', _('Add to Contact List…')),
         ('win.block-contact-', _('Block Contact…')),
-        ('win.toggle-audio-', _('Voice Chat')),
-        ('win.toggle-video-', _('Video Chat')),
+        ('win.start-call-', _('Start Call…')),
         ('win.information-', _('Information')),
         ('app.browse-history', _('History')),
-        ]
+    ]
 
     def build_chatstate_menu():
         menu = Gio.Menu()
@@ -547,6 +546,9 @@ def get_singlechat_menu(control_id, account, jid):
         for item in preset:
             if isinstance(item[1], str):
                 action_name, label = item
+                if action_name == 'win.send-marker-' and type_ == 'pm':
+                    continue
+
                 if action_name == 'app.browse-history':
                     menuitem = Gio.MenuItem.new(label, action_name)
                     dict_ = {'account': GLib.Variant('s', account),
@@ -572,6 +574,7 @@ def get_singlechat_menu(control_id, account, jid):
 def get_groupchat_menu(control_id, account, jid):
     groupchat_menu = [
         ('win.information-', _('Information')),
+        ('win.groupchat-settings-', _('Settings…')),
         (_('Manage Group Chat'), [
             ('win.rename-groupchat-', _('Rename…')),
             ('win.change-subject-', _('Change Subject…')),
@@ -579,16 +582,6 @@ def get_groupchat_menu(control_id, account, jid):
             ('win.configure-', _('Configure…')),
             ('win.destroy-', _('Destroy…')),
         ]),
-        (_('Chat Settings'), [
-            ('win.print-join-left-', _('Show Join/Leave')),
-            ('win.print-status-', _('Show Status Changes')),
-            ('win.notify-on-message-', _('Notify on all Messages')),
-            ('win.minimize-on-close-', _('Minimize on Close')),
-            ('win.minimize-on-autojoin-',
-             _('Minimize When Joining Automatically')),
-            (_('Send Chatstate'), ['chatstate']),
-        ]),
-        (_('Sync Threshold'), ['sync']),
         ('win.change-nickname-', _('Change Nickname…')),
         ('win.request-voice-', _('Request Voice')),
         ('win.execute-command-', _('Execute Command…')),
@@ -620,44 +613,10 @@ def get_groupchat_menu(control_id, account, jid):
                     menu.append(label, action_name + control_id)
             else:
                 label, sub_menu = item
-                if 'sync' in sub_menu:
-                    # Sync threshold menu
-                    submenu = build_sync_menu()
-                elif 'chatstate' in sub_menu:
-                    submenu = build_chatstate_menu()
-                else:
-                    # This is a submenu
-                    submenu = build_menu(sub_menu)
+                submenu = build_menu(sub_menu)
                 menu.append_submenu(label, submenu)
         return menu
 
-    def build_sync_menu():
-        menu = Gio.Menu()
-        days = app.config.get('threshold_options').split(',')
-        days = [int(day) for day in days]
-        action_name = 'win.choose-sync-%s::' % control_id
-        for day in days:
-            if day == 0:
-                label = _('No threshold')
-            else:
-                label = ngettext('%i day', '%i days', day, day, day)
-            menu.append(label, '%s%s' % (action_name, day))
-        return menu
-
-    def build_chatstate_menu():
-        menu = Gio.Menu()
-        entries = [
-            (_('Disabled'), 'disabled'),
-            (_('Composing only'), 'composing_only'),
-            (_('All chat states'), 'all')
-        ]
-
-        for entry in entries:
-            label, setting = entry
-            action = 'win.send-chatstate-%s::%s' % (control_id, setting)
-            menu.append(label, action)
-        return menu
-
     return build_menu(groupchat_menu)
 
 
